@@ -1,5 +1,9 @@
-import re
+from __future__ import annotations
 
+import re
+from typing import TYPE_CHECKING
+
+from aiolimiter import AsyncLimiter
 from yarl import URL
 
 from ..base_functions.base_functions import (
@@ -11,27 +15,51 @@ from ..base_functions.base_functions import (
 )
 from ..base_functions.data_classes import AlbumItem, MediaItem
 from ..base_functions.error_classes import NoExtensionFailure
-from ..base_functions.sql_helper import SQLHelper
-from ..client.client import ScrapeSession
+
+if TYPE_CHECKING:
+    from ..base_functions.base_functions import ErrorFileWriter
+    from ..base_functions.sql_helper import SQLHelper
+    from ..client.client import ScrapeSession
 
 
 class BunkrCrawler:
-    def __init__(self, quiet: bool, SQL_Helper: SQLHelper, remove_bunkr_id: bool):
+    def __init__(self, quiet: bool, SQL_Helper: SQLHelper, remove_bunkr_id: bool, error_writer: ErrorFileWriter):
         self.quiet = quiet
         self.SQL_Helper = SQL_Helper
         self.remove_bunkr_id = remove_bunkr_id
+        self.limiter = AsyncLimiter(20, 1)
+
+        self.error_writer = error_writer
+
+    async def get_stream_link(self, url: URL):
+        cdn_possibilities = r"(?:cdn.bunkrr...|media-files.bunkrr...|media-files..bunkrr...|media-files...bunkrr...|cdn.bunkr...|cdn..bunkr...|cdn...bunkr...|media-files.bunkr...|media-files..bunkr...|media-files...bunkr...)"
+        ext = '.' + url.parts[-1].split('.')[-1]
+        if ext:
+            ext = ext.lower()
+        else:
+            return url
+
+        if ext in FILE_FORMATS['Images']:
+            url = URL(str(url).replace("https://cdn", "https://i"))
+        elif ext in FILE_FORMATS['Videos']:
+            url = URL(re.sub(cdn_possibilities, "bunkrr.su/v", str(url)))
+        else:
+            url = URL(re.sub(cdn_possibilities, "bunkrr.su/d", str(url)))
+        return url
 
     async def fetch(self, session: ScrapeSession, url: URL) -> AlbumItem:
         """Scraper for Bunkr"""
         album_obj = AlbumItem("Loose Bunkr Files", [])
-        await log(f"Starting: {str(url)}", quiet=self.quiet, style="green")
+        log(f"Starting: {url}", quiet=self.quiet, style="green")
+
+        url = await self.get_stream_link(url)
 
         if "v" in url.parts or "d" in url.parts:
             media = await self.get_file(session, url)
             if not media.filename:
                 return album_obj
             await album_obj.add_media(media)
-            await log(f"Finished: {str(url)}", quiet=self.quiet, style="green")
+            log(f"Finished: {url}", quiet=self.quiet, style="green")
             if not media.complete:
                 await self.SQL_Helper.insert_media("bunkr", "", media)
             return album_obj
@@ -41,10 +69,9 @@ class BunkrCrawler:
             await self.SQL_Helper.insert_album("bunkr", url, album_obj)
 
             if album_obj.media:
-                await log(f"Finished: {str(url)}", quiet=self.quiet, style="green")
+                log(f"Finished: {url}", quiet=self.quiet, style="green")
             return album_obj
 
-        cdn_possibilities = r"(?:cdn.bunkr...|cdn..bunkr...|cdn...bunkr...|media-files.bunkr...|media-files..bunkr...|media-files...bunkr...)"
         ext = '.' + url.parts[-1].split('.')[-1]
         if ext:
             ext = ext.lower()
@@ -55,19 +82,14 @@ class BunkrCrawler:
             await self.SQL_Helper.fix_bunkr_entries(url, original_filename)
             check_complete = await self.SQL_Helper.check_complete_singular("bunkr", url)
 
-            url = URL(str(url).replace("https://cdn", "https://i"))
             media_item = MediaItem(url, url, check_complete, filename, ext, original_filename)
             await album_obj.add_media(media_item)
         else:
-            if ext in FILE_FORMATS['Videos']:
-                referer = URL(re.sub(cdn_possibilities, "bunkr.la/v", str(url)))
-            else:
-                referer = URL(re.sub(cdn_possibilities, "bunkr.la/d", str(url)))
-            media_item = await self.get_file(session, referer)
+            media_item = await self.get_file(session, url)
             await album_obj.add_media(media_item)
 
         await self.SQL_Helper.insert_album("bunkr", url, album_obj)
-        await log(f"Finished: {str(url)}", quiet=self.quiet, style="green")
+        log(f"Finished: {url}", quiet=self.quiet, style="green")
         return album_obj
 
     async def remove_id(self, filename: str, ext: str):
@@ -81,6 +103,7 @@ class BunkrCrawler:
         return original_filename, filename
 
     async def check_for_la(self, url: URL):
+        assert url.host is not None
         if "12" in url.host:
             url_host = url.host.replace(".su", ".la").replace(".ru", ".la")
             url = url.with_host(url_host)
@@ -90,10 +113,11 @@ class BunkrCrawler:
         """Gets the media item from the supplied url"""
 
         ### Temp Fix ###
-        url = url.with_host("bunkr.la")
+        url = url.with_host("bunkrr.su")
 
         try:
-            soup = await session.get_BS4(url)
+            async with self.limiter:
+                soup = await session.get_BS4(url)
             head = soup.select_one("head")
             scripts = head.select('script[type="text/javascript"]')
             link = None
@@ -117,8 +141,9 @@ class BunkrCrawler:
             return MediaItem(link, url, complete, filename, ext, original_filename)
 
         except Exception as e:
-            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
-            await log(f"Error: {str(url)}", quiet=self.quiet, style="red")
+            logger.debug("Error encountered while handling %s", url, exc_info=True)
+            log(f"Error: {url}", quiet=self.quiet, style="red")
+            await self.error_writer.write_errored_scrape(url, e, self.quiet)
             logger.debug(e)
             return MediaItem(url, url, False, "", "", "")
 
@@ -126,11 +151,12 @@ class BunkrCrawler:
         """Iterates through an album and creates the media items"""
 
         ### Temp Fix ###
-        url = url.with_host("bunkr.la")
+        url = url.with_host("bunkrr.su")
 
         album = AlbumItem(url.name, [])
         try:
-            soup = await session.get_BS4(url)
+            async with self.limiter:
+                soup = await session.get_BS4(url)
             title = soup.select_one('h1[class="text-[24px] font-bold text-dark dark:text-white"]')
             for elem in title.find_all("span"):
                 elem.decompose()
@@ -140,17 +166,18 @@ class BunkrCrawler:
                 link = file.get("href")
                 media_loc = file.select_one("img").get("src").split("//i")[-1].split(".bunkr.")[0]
 
+                assert url.host is not None
                 if link.startswith("/"):
                     link = URL("https://" + url.host + link)
                 link = URL(link)
+                referer = await self.get_stream_link(link)
 
                 try:
                     filename, ext = await get_filename_and_ext(link.name)
                 except NoExtensionFailure:
-                    logger.debug("Couldn't get extension for %s", str(link))
+                    logger.debug("Couldn't get extension for %s", link)
                     continue
 
-                referer = link
                 if "v" in link.parts or "d" in link.parts:
                     media = await self.get_file(session, link)
                     link = media.url
@@ -170,8 +197,7 @@ class BunkrCrawler:
                 await album.add_media(media)
 
         except Exception as e:
-            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
-            await log(f"Error: {str(url)}", quiet=self.quiet, style="red")
-            logger.debug(e)
+            logger.debug("Error encountered while handling %s", url, exc_info=True)
+            await self.error_writer.write_errored_scrape(url, e, self.quiet)
 
         return album
