@@ -1,33 +1,40 @@
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
 from yarl import URL
 
 from ..base_functions.base_functions import (
     check_direct,
-    get_filename_and_ext,
+    create_media_item,
     log,
     logger,
     make_title_safe,
 )
-from ..base_functions.data_classes import AlbumItem, MediaItem
+from ..base_functions.data_classes import AlbumItem
 from ..base_functions.error_classes import NoExtensionFailure
-from ..base_functions.sql_helper import SQLHelper
-from ..client.client import ScrapeSession
+
+if TYPE_CHECKING:
+    from ..base_functions.base_functions import ErrorFileWriter
+    from ..base_functions.sql_helper import SQLHelper
+    from ..client.client import ScrapeSession
 
 
 class ImgBoxCrawler:
-    def __init__(self, *, quiet: bool, SQL_Helper: SQLHelper):
+    def __init__(self, *, quiet: bool, SQL_Helper: SQLHelper, error_writer: ErrorFileWriter):
         self.quiet = quiet
         self.SQL_Helper = SQL_Helper
 
+        self.error_writer = error_writer
+
     async def fetch(self, session: ScrapeSession, url: URL) -> AlbumItem:
         """Director func for ImgBox scraping"""
         album_obj = AlbumItem("Loose ImgBox Files", [])
-        await log(f"Starting: {str(url)}", quiet=self.quiet, style="green")
+        log(f"Starting: {url}", quiet=self.quiet, style="green")
 
         try:
             if await check_direct(url):
-                filename, ext = await get_filename_and_ext(url.name)
-                complete = await self.SQL_Helper.check_complete_singular("imgbox", url)
-                media_item = MediaItem(url, url, complete, filename, ext, filename)
+                media_item = await create_media_item(url, url, self.SQL_Helper, "imgbox")
                 await album_obj.add_media(media_item)
 
             elif "g" in url.parts:
@@ -38,27 +45,22 @@ class ImgBoxCrawler:
                 await album_obj.set_new_title(title)
                 for img in images:
                     try:
-                        filename, ext = await get_filename_and_ext(img.name)
+                        media_item = await create_media_item(img, url, self.SQL_Helper, "imgbox")
                     except NoExtensionFailure:
-                        logger.debug("Couldn't get extension for %s", str(img))
+                        logger.debug("Couldn't get extension for %s", img)
                         continue
-                    complete = await self.SQL_Helper.check_complete_singular("imgbox", img)
-                    media_item = MediaItem(img, url, complete, filename, ext, filename)
                     await album_obj.add_media(media_item)
             else:
                 img = await self.singular(session, url)
-                filename, ext = await get_filename_and_ext(img.name)
-                complete = await self.SQL_Helper.check_complete_singular("imgbox", img)
-                media_item = MediaItem(img, url, complete, filename, ext, filename)
+                media_item = await create_media_item(img, url, self.SQL_Helper, "imgbox")
                 await album_obj.add_media(media_item)
 
         except Exception as e:
-            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
-            await log(f"Error: {str(url)}", quiet=self.quiet, style="red")
-            logger.debug(e)
+            logger.debug("Error encountered while handling %s", url, exc_info=True)
+            await self.error_writer.write_errored_scrape(url, e, self.quiet)
 
         await self.SQL_Helper.insert_album("imgbox", url, album_obj)
-        await log(f"Finished: {str(url)}", quiet=self.quiet, style="green")
+        log(f"Finished: {url}", quiet=self.quiet, style="green")
         return album_obj
 
     async def folder(self, session: ScrapeSession, url: URL):
@@ -75,8 +77,7 @@ class ImgBoxCrawler:
 
         return title, output
 
-    async def singular(self, session: ScrapeSession, url: URL):
+    async def singular(self, session: ScrapeSession, url: URL) -> URL:
         """Gets individual links"""
         soup = await session.get_BS4(url)
-        link = URL(soup.select_one("img[id=img]").get('src'))
-        return link
+        return URL(soup.select_one("img[id=img]").get('src'))
