@@ -1,19 +1,28 @@
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
 from yarl import URL
 
-from ..base_functions.base_functions import get_filename_and_ext, log, logger, make_title_safe
-from ..base_functions.data_classes import AlbumItem, MediaItem
-from ..base_functions.sql_helper import SQLHelper
-from ..client.client import ScrapeSession
+from ..base_functions.base_functions import create_media_item, log, logger, make_title_safe
+from ..base_functions.data_classes import AlbumItem
+
+if TYPE_CHECKING:
+    from ..base_functions.base_functions import ErrorFileWriter
+    from ..base_functions.sql_helper import SQLHelper
+    from ..client.client import ScrapeSession
 
 
 class PimpAndHostCrawler:
-    def __init__(self, quiet: bool, SQL_Helper: SQLHelper):
+    def __init__(self, quiet: bool, SQL_Helper: SQLHelper, error_writer: ErrorFileWriter):
         self.quiet = quiet
         self.SQL_Helper = SQL_Helper
 
+        self.error_writer = error_writer
+
     async def fetch(self, session: ScrapeSession, url: URL) -> AlbumItem:
         """Director for pimpandhost scraping"""
-        await log(f"Starting: {str(url)}", quiet=self.quiet, style="green")
+        log(f"Starting: {url}", quiet=self.quiet, style="green")
         album_obj = AlbumItem("Loose Pixeldrain Files", [])
 
         if url.parts[1] == 'album':
@@ -27,7 +36,7 @@ class PimpAndHostCrawler:
             await album_obj.add_media(media_item)
 
         await self.SQL_Helper.insert_album("pimpandhost", url, album_obj)
-        await log(f"Finished: {str(url)}", quiet=self.quiet, style="green")
+        log(f"Finished: {url}", quiet=self.quiet, style="green")
         return album_obj
 
     async def get_listings(self, session: ScrapeSession, url: URL):
@@ -41,14 +50,13 @@ class PimpAndHostCrawler:
             title = await make_title_safe(title.get_text())
 
             for file in soup.select('a[class*="image-wrapper center-cropped im-wr"]'):
-                link = file.get("href")
+                link = URL(file.get("href"))
                 media_items.append(await self.get_singular(session, link))
             return media_items, title
 
         except Exception as e:
-            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
-            await log(f"Error: {str(url)}", quiet=self.quiet, style="red")
-            logger.debug(e)
+            logger.debug("Error encountered while handling %s", url, exc_info=True)
+            await self.error_writer.write_errored_scrape(url, e, self.quiet)
 
     async def get_singular(self, session: ScrapeSession, url: URL):
         """Handles singular files"""
@@ -58,11 +66,7 @@ class PimpAndHostCrawler:
             img = img.get("src")
             if img.startswith("//"):
                 img = URL("https:" + img)
-            filename, ext = await get_filename_and_ext(img.name)
-            complete = await self.SQL_Helper.check_complete_singular("pimpandhost", img)
-            media_item = MediaItem(img, url, complete, filename, ext, filename)
-            return media_item
+            return await create_media_item(img, url, self.SQL_Helper, "pimpandhost")
         except Exception as e:
-            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
-            await log(f"Error: {str(url)}", quiet=self.quiet, style="red")
-            logger.debug(e)
+            logger.debug("Error encountered while handling %s", url, exc_info=True)
+            await self.error_writer.write_errored_scrape(url, e, self.quiet)
