@@ -5,17 +5,19 @@ import itertools
 import logging
 from http import HTTPStatus
 from random import gauss
+from typing import TYPE_CHECKING, Any, Dict
 
-import aiofiles
 import aiohttp.client_exceptions
 from tqdm import tqdm
 
 from cyberdrop_dl.base_functions.base_functions import (
+    FILE_FORMATS,
+    ErrorFileWriter,
     clear,
     log,
     logger,
 )
-from cyberdrop_dl.base_functions.data_classes import AlbumItem, CascadeItem, DomainItem, FileLock, ForumItem, MediaItem
+from cyberdrop_dl.base_functions.data_classes import AlbumItem, DomainItem, FileLock, ForumItem, MediaItem
 from cyberdrop_dl.base_functions.error_classes import DownloadFailure
 from cyberdrop_dl.base_functions.sql_helper import SQLHelper, get_db_path
 from cyberdrop_dl.client.client import Client, DownloadSession
@@ -30,6 +32,9 @@ from .downloader_utils import (
     retry,
 )
 
+if TYPE_CHECKING:
+    from pathlib import Path
+
 
 class Files:
     """Class that keeps track of completed, skipped and failed files"""
@@ -40,15 +45,15 @@ class Files:
         self.skipped_files = 0
         self.failed_files = 0
 
-    async def add_completed(self):
+    def add_completed(self) -> None:
         self.completed_files += 1
         self.progress.update(1)
 
-    async def add_skipped(self):
+    def add_skipped(self) -> None:
         self.skipped_files += 1
         self.progress.update(1)
 
-    async def add_failed(self):
+    def add_failed(self) -> None:
         self.failed_files += 1
         self.progress.update(1)
 
@@ -56,22 +61,20 @@ class Files:
 class Old_Downloader:
     """Downloader class, directs downloading for domain objects"""
 
-    def __init__(self, args: dict, client: Client, SQL_Helper: SQLHelper,
-                 domain: str, domain_obj: DomainItem, files: Files):
+    def __init__(self, args: Dict, client: Client, SQL_Helper: SQLHelper,
+                 domain: str, domain_obj: DomainItem, files: Files, error_writer: ErrorFileWriter):
         self.client = client
         self.download_session = DownloadSession(client)
+        self.error_writer = error_writer
         self.File_Lock = FileLock()
         self.SQL_Helper = SQL_Helper
 
         self.domain = domain
         self.domain_obj = domain_obj
 
-        self.errored_output = args['Runtime']['output_errored_urls']
-        self.errored_file = args['Files']['errored_urls_file']
-
         self.files = files
 
-        self.current_attempt = {}
+        self.current_attempt: Dict[str, int] = {}
         max_workers = get_threads_number(args, domain)
         self._semaphore = asyncio.Semaphore(max_workers)
         self.delay = {'cyberfile': 1, 'anonfiles': 1, "coomer": 0.2, "kemono": 0.2}
@@ -83,6 +86,13 @@ class Old_Downloader:
         self.exclude_videos = args["Ignore"]["exclude_videos"]
         self.exclude_other = args["Ignore"]["exclude_other"]
 
+        self.filesize_minimum_images = args["Runtime"]["filesize_minimum_images"]
+        self.filesize_minimum_videos = args["Runtime"]["filesize_minimum_videos"]
+        self.filesize_minimum_other = args["Runtime"]["filesize_minimum_other"]
+        self.filesize_maximum_images = args["Runtime"]["filesize_maximum_images"]
+        self.filesize_maximum_videos = args["Runtime"]["filesize_maximum_videos"]
+        self.filesize_maximum_other = args["Runtime"]["filesize_maximum_other"]
+
         self.block_sub_folders = args['Runtime']['block_sub_folders']
         self.allowed_attempts = args["Runtime"]["attempts"]
         self.disable_attempt_limit = args["Runtime"]["disable_attempt_limit"]
@@ -91,13 +101,13 @@ class Old_Downloader:
         self.proxy = args["Runtime"]["proxy"]
         self.required_free_space = args["Runtime"]["required_free_space"]
 
-    async def start_domain(self):
+    async def start_domain(self) -> None:
         """Handler for domains and the progress bars for it"""
         for album, album_obj in self.domain_obj.albums.items():
             await self.start_album(album, album_obj)
         await self.download_session.exit_handler()
 
-    async def start_album(self, album: str, album_obj: AlbumItem):
+    async def start_album(self, album: str, album_obj: AlbumItem) -> None:
         """Handler for albums and the progress bars for it"""
         if await album_obj.is_empty():
             return
@@ -106,11 +116,11 @@ class Old_Downloader:
             download_tasks.append(self.start_file(album, media))
         await asyncio.gather(*download_tasks)
 
-    async def start_file(self, album: str, media: MediaItem):
+    async def start_file(self, album: str, media: MediaItem) -> None:
         """Handler for files and the progress bars for it"""
         if media.complete or await self.SQL_Helper.check_complete_singular(self.domain, media.url):
-            await log(f"Previously Downloaded: {media.filename}", quiet=True)
-            await self.files.add_skipped()
+            log(f"Previously Downloaded: {media.filename}", quiet=True)
+            self.files.add_skipped()
             return
         async with self._semaphore:
             url_path = await get_db_path(media.url, self.domain)
@@ -120,14 +130,14 @@ class Old_Downloader:
     async def download_file(self, album: str, media: MediaItem, url_path: str) -> None:
         """File downloader"""
         if not await check_free_space(self.required_free_space, self.download_dir):
-            await log("We've run out of free space.", quiet=True)
-            await self.files.add_skipped()
+            log("We've run out of free space.", quiet=True)
+            self.files.add_skipped()
             return
 
         if not await allowed_filetype(media, self.exclude_images, self.exclude_videos, self.exclude_audio,
                                       self.exclude_other):
-            await log(f"Blocked by file extension: {media.filename}", quiet=True)
-            await self.files.add_skipped()
+            log(f"Blocked by file extension: {media.filename}", quiet=True)
+            self.files.add_skipped()
             return
 
         if self.block_sub_folders:
@@ -147,49 +157,50 @@ class Old_Downloader:
             complete_file = (self.download_dir / album / media.filename)
             partial_file = complete_file.with_suffix(complete_file.suffix + '.part')
 
-            fake_download = False
-            if self.mark_downloaded:
-                fake_download = True
-
-            complete_file, partial_file, proceed = await self.check_file_exists(complete_file, partial_file, media,
-                                                                                album, url_path, original_filename,
-                                                                                current_throttle)
-            if not proceed:
-                fake_download = True
+            complete_file, partial_file, proceed, expected_size = await self.check_file_exists(complete_file,
+                                                                                               partial_file, media,
+                                                                                               album, url_path,
+                                                                                               original_filename,
+                                                                                               current_throttle)
+            fake_download = self.mark_downloaded or not proceed
 
             await self.SQL_Helper.update_pre_download(complete_file, media.filename, url_path, original_filename)
 
-            resume_point = 0
-            await self.SQL_Helper.sql_insert_temp(str(partial_file))
-            range_num = None
-            if partial_file.exists():
-                resume_point = partial_file.stat().st_size
-                range_num = f'bytes={resume_point}-'
+            filesize_check = await self.check_filesize_limits(media, expected_size)
+            if not filesize_check:
+                log(f"Filesize out of range: {media.filename}", quiet=True)
+                self.files.add_skipped()
+                return
 
+            await self.SQL_Helper.sql_insert_temp(str(partial_file))
+            resume_point = partial_file.stat().st_size if partial_file.exists() else 0
+
+            assert media.url.host is not None
             for key, value in self.delay.items():
                 if key in media.url.host:
                     current_throttle = value
 
-            headers = {"Authorization": await basic_auth("Cyberdrop-DL", self.pixeldrain_api_key)} \
-                if (self.pixeldrain_api_key and "pixeldrain" in media.url.host) else {}
-            if range_num:
-                headers['Range'] = range_num
+            headers = {}
+            if self.pixeldrain_api_key and "pixeldrain" in media.url.host:
+                headers["Authorization"] = await basic_auth("Cyberdrop-DL", self.pixeldrain_api_key)
+
+            headers['Range'] = f'bytes={resume_point}-'
 
             if not await self.SQL_Helper.sql_check_old_existing(url_path) and not fake_download:
                 await self.download_session.old_download_file(media, partial_file, current_throttle, resume_point,
-                                                              self.proxy, headers)
+                                                              self.proxy, headers, expected_size)
                 partial_file.rename(complete_file)
 
             await self.SQL_Helper.mark_complete(url_path, original_filename)
-            if media.url.parts[-1] in self.current_attempt.keys():
+            if media.url.parts[-1] in self.current_attempt:
                 self.current_attempt.pop(media.url.parts[-1])
 
             if fake_download:
-                await self.files.add_skipped()
+                self.files.add_skipped()
             else:
-                await self.files.add_completed()
+                self.files.add_completed()
 
-            await log(f"Completed Download: {media.filename} from {media.referer}", quiet=True)
+            log(f"Completed Download: {media.filename} from {media.referer}", quiet=True)
             await self.File_Lock.remove_lock(original_filename)
             return
 
@@ -200,54 +211,79 @@ class Old_Downloader:
             if await self.File_Lock.check_lock(original_filename):
                 await self.File_Lock.remove_lock(original_filename)
 
-            new_error = DownloadFailure(code=1)
-
             if hasattr(e, "message"):
                 logging.debug(f"\n{media.url} ({e.message})")
-            new_error.message = repr(e)
 
             if hasattr(e, "code"):
                 if await is_4xx_client_error(e.code) and e.code != HTTPStatus.TOO_MANY_REQUESTS:
-                    logger.debug("We ran into a 400 level error: %s", str(e.code))
-                    await log(f"Failed Download: {media.filename}", quiet=True)
-                    await self.files.add_failed()
-                    if url_path in self.current_attempt.keys():
+                    logger.debug("We ran into a 400 level error: %s", e.code)
+                    log(f"Failed Download: {media.filename}", quiet=True)
+                    if url_path in self.current_attempt:
                         self.current_attempt.pop(url_path)
-                    await self.output_failed(media, e)
+                    await self.handle_failed(media, e)
                     return
                 if e.code == HTTPStatus.SERVICE_UNAVAILABLE or e.code == CustomHTTPStatus.WEB_SERVER_IS_DOWN:
                     if hasattr(e, "message"):
                         if not e.message:
                             e.message = "Web server is down"
                         logging.debug(f"\n{media.url} ({e.message})")
-                    await log(f"Failed Download: {media.filename}", quiet=True)
-                    await self.files.add_failed()
-                    if url_path in self.current_attempt.keys():
+                    log(f"Failed Download: {media.filename}", quiet=True)
+                    if url_path in self.current_attempt:
                         self.current_attempt.pop(url_path)
-                    await self.output_failed(media, e)
+                    await self.handle_failed(media, e)
                     return
-                logger.debug("Error status code: " + str(e.code))
-                new_error.code = e.code
+                logger.debug("Error status code: %s", e.code)
 
-            raise new_error
+            raise DownloadFailure(code=getattr(e, "code", 1), message=repr(e))
 
-    async def output_failed(self, media, e):
-        if self.errored_output:
-            async with aiofiles.open(self.errored_file, mode='a') as file:
-                await file.write(f"{media.url},{media.referer},{e.message}\n")
+    def can_retry(self, url_path: str) -> bool:
+        return self.disable_attempt_limit or self.current_attempt[url_path] < self.allowed_attempts - 1
 
-    async def check_file_exists(self, complete_file, partial_file, media, album, url_path, original_filename,
-                                current_throttle):
+    async def handle_failed(self, media: MediaItem, e: Any) -> None:
+        self.files.add_failed()
+        await self.error_writer.write_errored_download(media.url, media.referer, e.message)
+
+    async def check_filesize_limits(self, media: MediaItem, content_size: int) -> bool:
+        if media.ext in FILE_FORMATS['Images']:
+            if self.filesize_minimum_images and self.filesize_maximum_images:
+                if content_size < self.filesize_minimum_images or content_size > self.filesize_maximum_images:
+                    return False
+            if content_size < self.filesize_minimum_images:
+                return False
+            if self.filesize_maximum_images and content_size > self.filesize_maximum_images:
+                return False
+        elif media.ext in FILE_FORMATS['Videos']:
+            if self.filesize_minimum_videos and self.filesize_maximum_videos:
+                if content_size < self.filesize_minimum_videos or content_size > self.filesize_maximum_videos:
+                    return False
+            if content_size < self.filesize_minimum_videos:
+                return False
+            if self.filesize_maximum_videos and content_size > self.filesize_maximum_videos:
+                return False
+        else:
+            if self.filesize_minimum_other and self.filesize_maximum_other:
+                if content_size < self.filesize_minimum_other or content_size > self.filesize_maximum_other:
+                    return False
+            if content_size < self.filesize_minimum_other:
+                return False
+            if self.filesize_maximum_other and content_size > self.filesize_maximum_other:
+                return False
+        return True
+
+    async def check_file_exists(self, complete_file: Path, partial_file: Path, media: MediaItem, album: str,
+                                url_path: str, original_filename: str,
+                                current_throttle: float) -> tuple[Path, Path, bool, int]:
         """Complicated checker for if a file already exists, and was already downloaded"""
         expected_size = None
         proceed = True
         while True:
-            if not complete_file.exists() and not partial_file.exists():
-                break
-
             if not expected_size:
                 expected_size = await self.download_session.get_filesize(media.url, str(media.referer),
                                                                          current_throttle)
+
+            if not complete_file.exists() and not partial_file.exists():
+                break
+
             if complete_file.exists() and complete_file.stat().st_size == expected_size:
                 proceed = False
                 break
@@ -276,27 +312,11 @@ class Old_Downloader:
             complete_file = (self.download_dir / album / media.filename)
             partial_file = complete_file.with_suffix(complete_file.suffix + '.part')
 
-        return complete_file, partial_file, proceed
+        return complete_file, partial_file, proceed, expected_size
 
 
-async def old_download_cascade(args: dict, Cascade: CascadeItem, SQL_Helper: SQLHelper, client: Client) -> None:
-    """Handler for cascades and the progress bars for it"""
-    total_files = await Cascade.get_total()
-    with tqdm(total=total_files, unit_scale=True, unit='Files', leave=True, initial=0,
-              desc="Files Downloaded") as progress:
-        files = Files(progress)
-        tasks = []
-        for domain, domain_obj in Cascade.domains.items():
-            downloader = Old_Downloader(args, client, SQL_Helper, domain, domain_obj, files)
-            tasks.append(downloader.start_domain())
-        await asyncio.gather(*tasks)
-
-    await clear()
-    await log(f"| [green]Files Complete: {files.completed_files}[/green] - [yellow]Files Skipped: "
-              f"{files.skipped_files}[/yellow] - [red]Files Failed: {files.failed_files}[/red] |")
-
-
-async def old_download_forums(args: dict, Forums: ForumItem, SQL_Helper: SQLHelper, client: Client) -> None:
+async def old_download_forums(args: Dict, Forums: ForumItem, SQL_Helper: SQLHelper, client: Client,
+                              error_writer: ErrorFileWriter) -> None:
     """Handler for forum threads and the progress bars for it"""
     total_files = await Forums.get_total()
     with tqdm(total=total_files, unit_scale=True, unit='Files', leave=True, initial=0,
@@ -305,10 +325,10 @@ async def old_download_forums(args: dict, Forums: ForumItem, SQL_Helper: SQLHelp
         for title, Cascade in Forums.threads.items():
             tasks = []
             for domain, domain_obj in Cascade.domains.items():
-                downloader = Old_Downloader(args, client, SQL_Helper, domain, domain_obj, files)
+                downloader = Old_Downloader(args, client, SQL_Helper, domain, domain_obj, files, error_writer)
                 tasks.append(downloader.start_domain())
             await asyncio.gather(*tasks)
 
     await clear()
-    await log(f"| [green]Files Complete: {files.completed_files}[/green] - [yellow]Files Skipped: "
-              f"{files.skipped_files}[/yellow] - [red]Files Failed: {files.failed_files}[/red] |")
+    log(f"| [green]Files Complete: {files.completed_files}[/green] - [yellow]Files Skipped: "
+        f"{files.skipped_files}[/yellow] - [red]Files Failed: {files.failed_files}[/red] |")
